---
title: "Evaluation Parameter Store (Metric Store)"
---
import UniversalMap from '/docs/images/universal_map/_universal_map.mdx';
import TechnicalTag from '../term_tags/_tag.mdx';
import SetupHeader from '/docs/images/universal_map/_um_setup_header.mdx'
import CreateHeader from '/docs/images/universal_map/_um_create_header.mdx';
import ValidateHeader from '/docs/images/universal_map/_um_validate_header.mdx';


<UniversalMap setup='active' connect='inactive' create='active' validate='active'/> 

## Overview

### Definition

An Evaluation Parameter Store is a connector to store and retrieve information about parameters used during Validation of an Expectation which reference simple expressions or previously generated metrics.

### Features and promises

Evaluation Parameter Stores provide a method of storing and retrieving the Metrics associated with Evaluation Parameters without also storing the additional information that accompanies Validation Results.

### Relationship to other objects

Evaluation Parameter Stores are an alternative way to store Metrics without the accompanying Validation Results.  The process of storing information in an Evaluation Parameter Store is also typically executed by an Action in a Checkpoint's `action_list`.  Metrics stored in Evaluation Parameter Stores are also available as Evaluation Parameters when defining Expectations.

## Use cases

<SetupHeader/>

When you initialize your Data Context for the first time, a configuration for an in-memory Evaluation Parameter Store will automatically be added to `great_expectations.yml`. You may change this configuration to work with different environments in the same fashion as you would configure any other store, by adding a `store_backend` key to the Evaluation Parameter Store's entry.

<CreateHeader/>

When some Expectations are created they require you to define an Evaluation Parameter.  One of the ways this can be done is to define a `'$PARAMETER': URN` pair in the Expectation's `value` dictionary.  When Great Expectations encounters a $PARAMETER flag during Validation, it will replace the `URN` with a value retrieved from an Evaluation Parameter Store or Metric Store.

<ValidateHeader/>

If an Evaluation Parameter is used in an Expectation that is being Validated, and the Evaluation Parameter is configured to reference a value in your Evaluation Parameter Store, the Evaluation Parameter Store will be used behind the scenes to retrieve that value.  You will not need to directly interact with it.

If a Checkpoint is run,and it contains the `StoreEvaluationParametersAction` Action in its `action_list`, then your Evaluation Parameter Store will be used to store the Evaluation Parameters that were generated by the Checkpoint as well as their values.

## Features

### Referencing prior results

The Evaluation Parameter Store permits you to store the Metrics generated by Evaluation Parameters when Expectations containing them are Validated.  It also allows you to retrieve these Evaluation Parameters and values at a later point.  This permits you to create Expectations that rely on comparing the data of your current Batch with the metrics generated from a previous Batch of data.

### Convenience of use

Reading from and writing to the Evaluation Parameter store is handled behind the scenes.  All you need to do is provide the correct URN path to the parameter you want to retrieve, or include the `StoreEvaluationParametersAction` Action in the `action_list` of any Checkpoint that generates Metrics from Evaluation Parameters that you want to store.

## API basics

### How to access

The Evaluation Parameter Store uses an URN schema for identifying dependencies between Expectation Suites.  A valid URN must begin with the string: `urn:great_expectations`.  Valid URNs for Evaluation Parameter Stores must have one of the following structures to be recognized by your Data Context:

```yaml title: "Replace names in <> with the desired name."
urn:great_expectations:validations:<expectation_suite_name>:<metric_name>
urn:great_expectations:validations:<expectation_suite_name>:<metric_name>:<metric_kwargs_id>
```
